#import "CustomWindow.h"
#import <AppKit/AppKit.h>
#include "math.h"

@implementation CustomWindow

@synthesize initialLocation;
@synthesize offsetW, offsetH, screenW, screenH, w, h;
@synthesize boffsetW, boffsetH, bscreenW, bscreenH, bw, bh;
@synthesize scale;
@synthesize glView;

- (id)initWithResolution: (int) resolution {
    switch(resolution) {
        case WSVGA:
            bw = 740; bh = 1164;
            bscreenW = 600; bscreenH = 1024;
            boffsetW = 70; boffsetH = 70;
            imageName = @"wsvga";
            break;
        case XGA:
            bw = 808; bh = 1144;
            bscreenW = 768; bscreenH = 1024;
            boffsetW = 20; boffsetH = 60;
            imageName = @"xga";
            break;
        case FWVGA:
            bw = 520; bh = 994;
            bscreenW = 480; bscreenH = 854;
            boffsetW = 20; boffsetH = 70;
            imageName = @"fwvga";
            break;
        case WVGA:
            bw = 520; bh = 940;
            bscreenW = 480; bscreenH = 800;
            boffsetW = 20; boffsetH = 70;
            imageName = @"wvga";
            break;
        case WXGA:
            bw = 900; bh = 1380;
            bscreenW = 800; bscreenH = 1280;
            boffsetW = 50; boffsetH = 50;
            imageName = @"wxga";
            break;
        case QHD:
            bw = 580; bh = 1160 ;
            bscreenW = 540; bscreenH = 960;
            boffsetW = 20; boffsetH = 100;
            imageName = @"qhd";
            break;
        case DVGA:
            bw = 680; bh = 1080;
            bscreenW = 640; bscreenH = 960;
            boffsetW = 20; boffsetH = 60;
            imageName = @"dvga";
            break;
        case WDVGA:
            bw = 680; bh = 1256;
            bscreenW = 640; bscreenH = 1136;
            boffsetW = 20; boffsetH = 60;
            imageName = @"wdvga";
            break;
        case HVGA:
            bw = 350; bh = 600;
            bscreenW = 320; bscreenH = 480;
            boffsetW = 15; boffsetH = 60;
            imageName = @"hvga";
            break;
        default:
            bw = 350; bh = 600;
            bscreenW = 320; bscreenH = 480;
            boffsetW = 15; boffsetH = 60;
            imageName = @"hvga";
            break;
    }
    /*
     int minScreenSize = MIN([[NSScreen mainScreen] frame].size.width, [[NSScreen mainScreen] frame].size.height);
     int maxSimSize = MAX(w,h);
     
     scale = 1.0;
     while (maxSimSize > (minScreenSize - 50)) {
     scale -= 0.05;
     int testW = w * scale;
     int testH = h * scale;
     maxSimSize = MAX(testW, testH);
     }
     
     w *= scale;
     h *= scale;
     screenH *= scale;
     screenW *= scale;
     offsetH *= scale;
     offsetW *= scale;
     */
    [self scale:kPortrait];
    
    int centerX = [[NSScreen mainScreen] frame].size.width / 2;
    int centerY = [[NSScreen mainScreen] frame].size.height / 2;
    
    int x = centerX - (w / 2);
    int y = centerY - (h / 2);
    
    NSRect rect = NSMakeRect(x, y, w, h);
    self = [super initWithContentRect:rect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
    
    [self setBackgroundColor:[NSColor clearColor]];
    [self setOpaque:NO];
    image = [[NSImageView alloc] initWithFrame:rect];
    NSString *newImageName = [NSString stringWithFormat:@"%@p1.png", imageName];
    [image setImage:[NSImage imageNamed:newImageName]];
    [self setContentView:image];
    
    rect = NSMakeRect(0, 0, screenW, screenH);
    glView = [[EAGLView alloc] initWithFrame:rect];
    [glView setFrameOrigin:NSMakePoint(offsetW, offsetH)];
    [image addSubview:glView];
    
    return self;
}

- (void) scale:(int)orientation {
    scale = 1.0;
    
    int testW, testH;
    
    if (orientation == kLandscapeLeft || orientation == kLandscapeRight) {
        testW = bh;
        testH = bw;
        
        while (testW > ([[NSScreen mainScreen] frame].size.width - 50)) {
            scale -= 0.05;
            testW = bh * scale;
            testH = bw * scale;
        }
        while (testH > ([[NSScreen mainScreen] frame].size.height - 50)) {
            scale -= 0.05;
            testH = bw * scale;
        }
    } else {
        testW = bw;
        testH = bh;
        
        while (testW > ([[NSScreen mainScreen] frame].size.width - 50)) {
            scale -= 0.05;
            testW = bw * scale;
            testH = bh * scale;
        }
        while (testH > ([[NSScreen mainScreen] frame].size.height - 50)) {
            scale -= 0.05;
            testH = bh * scale;
        }
    }    
    
    w = bw * scale;
    h = bh * scale;
    screenH = bscreenH * scale;
    screenW = bscreenW * scale;
    offsetH = boffsetH * scale;
    offsetW = boffsetW * scale;
}

- (void) setScreenScale:(float) newScale {
    w = bw * newScale;
    h = bh * newScale;
    screenH = bscreenH * newScale;
    screenW = bscreenW * newScale;
    offsetH = boffsetH * newScale;
    offsetW = boffsetW * newScale;
}

-(void)redrawWindow:(int)orientation {
    [self scale:orientation];
    [self redrawWindow:orientation withScale:scale];
}


-(void)redrawWindow:(int)orientation withScale:(float) newScale{
    NSString *newImageName;
    
    w = bw * newScale;
    h = bh * newScale;
    screenH = bscreenH * newScale;
    screenW = bscreenW * newScale;
    offsetH = boffsetH * newScale;
    offsetW = boffsetW * newScale;
    
    int x = [self frame].origin.x;
    int y = [self frame].origin.y;
    
    int centerX = x + ([self frame].size.width / 2);
    int centerY = y + ([self frame].size.height / 2);
    
    if (orientation == kLandscapeLeft) {
        x = centerX - (h / 2);
        y = centerY - (w / 2);
        NSRect rect = NSMakeRect(x, y, h, w);
        [self setFrame:rect display:true];
        
        rect = NSMakeRect(0, 0, h, w);
        [image setFrame:rect];
        [image setFrameOrigin:NSMakePoint(0,0)];
        newImageName = [NSString stringWithFormat:@"%@l1.png", imageName];
        [image setImage:[NSImage imageNamed:newImageName]];
        
        [glView setFrameOrigin:NSMakePoint(offsetH, offsetW)];
        [self display];
    } else if (orientation == kLandscapeRight) {
        x = centerX - (h / 2);
        y = centerY - (w / 2);
        NSRect rect = NSMakeRect(x, y, h, w);
        [self setFrame:rect display:true];
        
        rect = NSMakeRect(0, 0, h, w);
        [image setFrame:rect];
        [image setFrameOrigin:NSMakePoint(0,0)];
        newImageName = [NSString stringWithFormat:@"%@l2.png", imageName];
        [image setImage:[NSImage imageNamed:newImageName]];
        
        [glView setFrameOrigin:NSMakePoint(offsetH, offsetW)];
        [self display];
    } else if (orientation == kPortrait) {
        x = centerX - (w / 2);
        y = centerY - (h / 2);
        NSRect rect = NSMakeRect(x, y, w, h);
        [self setFrame:rect display:true];
        
        rect = NSMakeRect(0, 0, w, h);
        [image setFrame:rect];
        [image setFrameOrigin:NSMakePoint(0,0)];
        newImageName = [NSString stringWithFormat:@"%@p1.png", imageName];
        [image setImage:[NSImage imageNamed:newImageName]];
        
        [glView setFrameOrigin:NSMakePoint(offsetW, offsetH)];
        [self display];
    } else if (orientation == kPortraitUpsideDown) {
        x = centerX - (w / 2);
        y = centerY - (h / 2);
        NSRect rect = NSMakeRect(x, y, w, h);
        [self setFrame:rect display:true];
        
        rect = NSMakeRect(0, 0, w, h);
        [image setFrame:rect];
        [image setFrameOrigin:NSMakePoint(0,0)];
        newImageName = [NSString stringWithFormat:@"%@p2.png", imageName];
        [image setImage:[NSImage imageNamed:newImageName]];
        
        [glView setFrameOrigin:NSMakePoint(offsetW, offsetH)];
        [self display];
    }
}


- (id)initWithContentRect:(NSRect)contentRect
                styleMask:(NSUInteger)aStyle
                  backing:(NSBackingStoreType)bufferingType
                    defer:(BOOL)flag {
    // Using NSBorderlessWindowMask results in a window without a title bar.
    self = [super initWithContentRect:contentRect styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
    if (self != nil) {
        // Start with no transparency for all drawing into the window
        [self setAlphaValue:1.0];
        // Turn off opacity so that the parts of the window that are not drawn into are transparent.
        [self setOpaque:NO];
    }
    return self;
}

/*
 Custom windows that use the NSBorderlessWindowMask can't become key by default. Override this method
 so that controls in this window will be enabled.
 */
- (BOOL)canBecomeKeyWindow {
    return YES;
}

/*
 Start tracking a potential drag operation here when the user first clicks the mouse, to establish
 the initial location.
 */
- (void)mouseDown:(NSEvent *)theEvent {
    // Get the mouse location in window coordinates.
    self.initialLocation = [theEvent locationInWindow];
}

/*
 Once the user starts dragging the mouse, move the window with it. The window has no title bar for
 the user to drag (so we have to implement dragging ourselves)
 */
- (void)mouseDragged:(NSEvent *)theEvent {
    NSRect screenVisibleFrame = [[NSScreen mainScreen] visibleFrame];
    NSRect windowFrame = [self frame];
    NSPoint newOrigin = windowFrame.origin;
    
    // Get the mouse location in window coordinates.
    NSPoint currentLocation = [theEvent locationInWindow];
    // Update the origin with the difference between the new mouse location and the old mouse location.
    newOrigin.x += (currentLocation.x - initialLocation.x);
    newOrigin.y += (currentLocation.y - initialLocation.y);
    
    // Don't let window get dragged up under the menu bar
    if ((newOrigin.y + windowFrame.size.height) > (screenVisibleFrame.origin.y + screenVisibleFrame.size.height)) {
        newOrigin.y = screenVisibleFrame.origin.y + (screenVisibleFrame.size.height - windowFrame.size.height);
    }
    
    // Move the window to the new location
    [self setFrameOrigin:newOrigin];
}

@end
